package de.ullisroboterseite.ursai2playeraddon;

// Autor: http://UllisRoboterSeite.de

// Doku:  http://UllisRoboterSeite.de/android-AI2-PlayerAddon.html
// Created: 2024-10-13
//
// Version 1 (2024-10-13)
// -------------------------
// - Basis-Version
//

import com.google.appinventor.components.annotations.*;
import com.google.appinventor.components.common.*;
import com.google.appinventor.components.runtime.*;

import android.media.MediaPlayer;
import android.os.Build;

import java.lang.reflect.*;
import java.util.*;

@DesignerComponent(version = 2 //
        , versionName = UrsPlayerAddon.VersionName //
        , dateBuilt = UrsPlayerAddon.dateBuilt //
        , description = "Addon to player component." //
        , category = com.google.appinventor.components.common.ComponentCategory.EXTENSION //
        , nonVisible = true //
        , helpUrl = "http://UllisRoboterSeite.de/android-AI2-PlayerAddon.html" //
        , iconName = "aiwebres/icon.png")
@SimpleObject(external = true)
@UsesPermissions(permissionNames = "")
public class UrsPlayerAddon extends AndroidNonvisibleComponent implements Component {
    static final String LOG_TAG = "PLAYER";
    static final String VersionName = "1.0.0";
    static final String dateBuilt = "2024-10-13";

    Player playerComponent;
    private MediaPlayer mp;

    public UrsPlayerAddon(ComponentContainer container) {
        super(container.$form());
    }

    @SimpleProperty(description = "Returns the component's version name.")
    public String Version() {
        return VersionName;
    }

    @SimpleProperty(description = "Returns running Android SDK version.")
    public int VersionSDK() {
        return Build.VERSION.SDK_INT;
    }

    @SimpleProperty(description = "Returns wether the app is running in the companion.")
    public boolean IsRunningInCompanion() {
        return form instanceof ReplForm;
    }

    @DesignerProperty(editorType = PropertyTypeConstants.PROPERTY_TYPE_COMPONENT, defaultValue = "")
    @SimpleProperty(category = PropertyCategory.GENERAL, description = "Player component to extend", userVisible = false)
    public void PlayerToUse(Component Player) {
        if (Player instanceof Player) {
            playerComponent = (Player) Player;
            try {
                Field f = Player.getClass().getDeclaredField("player"); //NoSuchFieldException
                f.setAccessible(true);
                mp = (MediaPlayer) f.get(playerComponent); //IllegalAccessException
            } catch (Exception e) {
                form.ErrorOccurred(this, "PlayerToUse", 18041, "PlayerToUse: Cannot access internal field 'player'");
            }
        } else {
            form.ErrorOccurred(this, "PlayerToUse", 18040, "PlayerToUse: Component not of type 'Player'");
            return;
        }
    }

    @SimpleProperty(category = PropertyCategory.BEHAVIOR, description = "Returns the current position of the currently playing audio file.")
    public int CurrentPosition() {
        if (mp != null)
            return mp.getCurrentPosition();
        else
            return -1;
    }

    @SimpleProperty(category = PropertyCategory.BEHAVIOR, description = "Returns the duration of the currently playing audio file.")
    public int Duration() {
        if (mp != null)
            return mp.getDuration();
        else
            return -1;
    }

    @SimpleFunction(description = "Seeks to specified time position.")
    public void SeekTo(int MilliSeconds) {
        if (mp != null)
            mp.seekTo(MilliSeconds);
    }

    @SimpleFunction(description = "Start at a given position.")
    public void StartAt(int MilliSeconds) {
        if (mp != null)
            mp.seekTo(MilliSeconds);
        playerComponent.Start();
    }
}